import os
import logging

from turbogears import controllers, expose, error_handler, validate
from turbogears import flash, redirect, url

from netborderPyInfra import PyGlobalConfig as globalConfig
from paraxip.gateway_gui import extjs
from paraxip.gateway_gui_widgets.wizardform import WizardForm
from status import StatusController

log = logging.getLogger("pstnconfig")

class WizardController(controllers.Controller):
  def __init__(self, in_strProjectPath):
   strSrcFile = os.path.join(in_strProjectPath, "pstnconfig/static/javascript/"
                             "widgets_src/wizard_form_src.js")
    
   self.wizard_form = WizardForm('wizardForm',
      title="Wizard",
              src_file=strSrcFile)

  def getWizardForm (self):    
    return self.wizard_form

  @expose()
  def index(self, tg_errors=None):
    """Show the pstnconfig form."""

    if tg_errors:
      flash('There was a problem with the form!')
      #isdnForm = isdn_wizard_form
    return dict(wizardForm=wizard_form)


    ''' -- 'save' controller method --
     Configuration obtained from wizard sent under 'boardConfig' and 
       'physicalConfig' keys in JSON format
       
     Examples:
    'boardConfigs': u'[{"board_id":1,
                       "model":"A108",
                       "num_interfaces":4,
                       "wanpipe":1}]'
    'physicalConfig': u'{"interface_type":"T1",
                         "framing_T1":"ESF",
                         "encoding_T1":"B8ZS",
                         "isdn_sw_T1":"5ESS",
                         "sip_url":"sip:localhost:5060",
                         "sip_port":"5066",
                         "transport_protocol":"udp"}',       
    '''
  @expose() # Invoked by Ext JS Form "submit": JSON response expected
  @error_handler(index)
  def save(self, **data):
    """Handle submission from the comment form and save the comment."""

    log.info("Data values from POST request: %s" % data)

    import pstnconfig.model as model
    model.currentConfig = model.createDefaultSangomaPstnConfig()

    digitalBoards = []
    analogBoards = []
    virtualDigitalBoards = []

    import simplejson
    # 'boardConfigs' key value is a JSON list
    for boardConfig in simplejson.loads(data['boardConfigs']):
      # TODO make sure that board IDs will always be in order
      # TODO using the same key names for the js and the Python would make 
      #   it easier
      boardModel = boardConfig['model']
      # Make sure is a known model
      assert model.boardsModelInfo.has_key(boardModel)
      boardModelInfo = model.boardsModelInfo[boardModel]
      if boardModelInfo['type'] == 'digital':
        digitalBoards.append(
          model.addSangomaDigitalBoard(model.currentConfig,
            boardModel=boardModel,
            boardID=boardConfig['board_id'],
            starting_wanpipe=boardConfig['wanpipe']))
      elif boardModelInfo['type'] == 'analog':
        analogBoards.append(
          model.addSangomaAnalogBoard(model.currentConfig,
            boardModel=boardModel,
            boardID=boardConfig['board_id'],
            wanpipe=boardConfig['wanpipe'],
            numInterfaces=boardConfig['num_interfaces']))
      elif boardModelInfo['type'] == 'virtual_digital':
        virtualDigitalBoards.append(
          model.addSangomaVirtualDigitalBoard(model.currentConfig,
            boardID=boardConfig['board_id'],
            numInterfaces=boardConfig['num_interfaces']))
      else:
        assert 'Unsupported board model type'

    # Add physical config to the model
    physicalConfig = simplejson.loads(data['physicalConfig'])
    interfaceType = physicalConfig['interface_type']

    if len(digitalBoards):

      if interfaceType == "T1":
        #Create the config parameters dictionary
        T1ConfigParams = {}
        T1ConfigParams['lineEncoding'] = physicalConfig['encoding_%s' % interfaceType]
        T1ConfigParams['clocking'] = 'TERMINAL'
        T1ConfigParams['loopbackMode'] = 'DISABLED'
        T1ConfigParams['framing'] = physicalConfig['framing_%s' % interfaceType]
        T1ConfigParams['LBO'] = '0dB'
        
        #Create the audio parameters dictionary
        T1AudioParams = {}
        T1AudioParams['defaultPcmLaw'] = "PCMU"
        T1AudioParams['voicePacketLengthInMs'] = "20"
        T1AudioParams['toneSetName'] = physicalConfig['country']

        digitalConfig = model.addT1SangomaDigitalConfig(model.currentConfig,
            'P%s-digital_%s' % (model.getNextPhysConfID(model.currentConfig), interfaceType),
            T1ConfigParams,
            T1AudioParams)

        for board in digitalBoards:
          model.applyConfigToBoardInterfaces(digitalConfig, board)

        # Indicate FAS config applies to all channels of all digital boards
        BChannelRange = model.BChannelRange.factory(firstChannel=1, 
            lastChannel=23, channelGroupID='default')

      elif interfaceType == "E1":
        #Create the config parameters dictionary
        E1ConfigParams = {}
        E1ConfigParams['lineEncoding'] = physicalConfig['encoding_%s' % interfaceType]
        E1ConfigParams['E1Signaling'] = 'CCS'
        E1ConfigParams['clocking'] = 'TERMINAL'
        E1ConfigParams['loopbackMode'] = 'DISABLED'
        E1ConfigParams['framing'] = physicalConfig['framing_%s' % interfaceType]
        E1ConfigParams['E1Termination'] = '120ohm'

        #Create the audio parameters dictionary
        E1AudioParams = {}
        E1AudioParams['defaultPcmLaw'] = "PCMA"
        E1AudioParams['voicePacketLengthInMs'] = "20"
        E1AudioParams['toneSetName'] = physicalConfig['country']

        digitalConfig = model.addE1SangomaDigitalConfig(model.currentConfig,
            'P%s-digital_%s' % (model.getNextPhysConfID(model.currentConfig),
            interfaceType), E1ConfigParams, E1AudioParams)

        for board in digitalBoards:
          model.applyConfigToBoardInterfaces(digitalConfig, board)

        # Indicate FAS config applies to all channels of all digital boards
        BChannelRange = model.BChannelRange.factory(firstChannel=1, 
            lastChannel=30, channelGroupID='default')

      fasConfig = model.addFasConfig(model.currentConfig,
          displayName='FAS1-%s' % interfaceType,
          termination='TERMINAL',
          switchVariant=physicalConfig['isdn_sw_%s' % interfaceType],
          BChannelNegotiation='EXCLUSIVE',
          initiateRestartProcedure='YES',
          inbandProgressTonesGeneration='AS-NEEDED',
          inbandProgressTonesIndicator='PROGRESS',
          callerNameLocation='IN-SETUP-MSG')

      for board in digitalBoards:
        model.applyConfigToBoardInterfaces(fasConfig, board, [BChannelRange])

    # Analog config
    if len(analogBoards):

      variant = physicalConfig['country']
      if variant == "TBR21":
        defaultPcmLaw = "PCMA"
      else:
        defaultPcmLaw = "PCMU"

      AudioParams = {}
      AudioParams['defaultPcmLaw'] = defaultPcmLaw
      AudioParams['voicePacketLengthInMs'] = "20"
      AudioParams['toneSetName'] = physicalConfig['country']

      analogConfig = model.addSangomaAnalogConfig(model.currentConfig,
          'P%s-analog' % model.getNextPhysConfID(model.currentConfig),
          variant,
          AudioParams)

      # Analog physical config applies to each analog board
      for board in analogBoards:
        model.applyConfigToBoard(analogConfig, board)

      configParams = {}
      configParams['displayName'] = 'FXO-config1'

      analogFxoConfig = model.addAnalogFxoConfig(model.currentConfig,
          configParams)
      # Analog FXO config applies to each interface of each analog board
      for board in analogBoards:
        model.applyConfigToBoardInterfaces(analogFxoConfig, board, 
            channelGroupID='default')

    # Virtual digital board config
    if len(virtualDigitalBoards):
      
      # Creates a dummy T1 physical config
      # Pretty much all physical config ignored with virtual boards, 
      #   but must still be provided (default provided)
      physicalDigitalConfig = model.addT1SangomaDigitalConfig(
          model.currentConfig,
          'Virtual Digital Config',
          model.defaultT1Config,
          model.defaultAudioParams)

      # Virtual boards must have one set of interfaces using TERMINAL 
      #   termination peered to interfaces using NETWORK termination
      fasConfigParams = model.defaultFasConfig

      fasConfigParams['displayName'] = 'Terminal'
      fasConfigParams['termination'] = 'TERMINAL'

      fasTerminalConfig = model.addFasConfig(model.currentConfig,
          **fasConfigParams)

      fasConfigParams['displayName'] = 'Network'
      fasConfigParams['termination'] = 'NETWORK'
      fasNetworkConfig = model.addFasConfig(model.currentConfig,
          **fasConfigParams)

      BChannelRange = model.BChannelRange.factory(firstChannel=1, 
          lastChannel=23, channelGroupID='default')

      for board in virtualDigitalBoards:
        # Physical config applies to all interfaces of virtual digital board
        model.applyConfigToBoardInterfaces(physicalDigitalConfig, board)

        half = len(board.get_interface()) /2
        
        # Apply TERMINAL config to first half of interfaces
        for interface in board.get_interface()[:half]:
          model.applyConfigToInterface(fasTerminalConfig, board,
              interface, [BChannelRange])

        # Apply NETWORK config to second half of interfaces
        for interface in board.get_interface()[half:]:
          model.applyConfigToInterface(fasNetworkConfig, board,
              interface, [BChannelRange])


    # Done with boards configuration

    # Write current config to file using default file location
    model.writeCurrentConfigToFile()
  
    # Update routing rules and gw.properties.        
    self.replaceTags("data/templates/T_routing-rules.xml", "config/routing-rules.xml", 
                     "XXX_DEFAULT_SIP_URL_XXX", physicalConfig['sip_url'])

    self.replaceTags("config/routing-rules.xml", "config/routing-rules.xml", 
                      "XXX_DEFAULT_SIP_TRANSPORT_XXX", physicalConfig['transport_protocol'])

    self.replaceTags("data/templates/T_gw.properties", "config/gw.properties", 
                     "XXX_DEFAULT_SIP_PORT_XXX", physicalConfig['sip_port'])
  
    # Invoked by Ext JS Form "submit": JSON response expected
    return(str(extjs.FormActionSubmitResponse(success=True)))      

  def replaceTags(self, in_strTemplateFileName, in_strFilename, in_strPattern, in_strValue):    
    """Finds all the occurence of the in_strPattern we need to replace in
       the file and change them for the corresponding value in_strValue"""

    log.info("Replacing tag [%s] with tag [%s]" % (in_strPattern,in_strValue))
    # Open file
    templates_path = globalConfig.getParameter("netborder.Installation.Directory")

    if templates_path is None:
      log.error("netborder.Installation.Directory parameter not set. Cannot "\
                 "change value in file %s", in_strFilename)
      return

    strTemplateFullFileName = os.path.join(templates_path, in_strTemplateFileName)
    strFullFileName = os.path.join(templates_path, in_strFilename)

    if not os.path.isfile(strTemplateFullFileName):
      log.error("Cannot find file %s", strTemplateFullFileName)               
      return     

    if not os.path.isfile(strFullFileName):
      log.error("Cannot find file %s", strFullFileName)               
      return     

    # Open template file and get its content.
    try:
      f = open(strTemplateFullFileName,'r')
      # Read all content
      strFileContent = f.read()
      f.close()
    except IOError:
      log.error("Cannot open file %s", strTemplateFullFileName)               
      return     

    if strFileContent == "":
      log.error("Error file %s is empty",strTemplateFullFileName)
  
    # Replace string in file file.  
    strFileContent=strFileContent.replace(in_strPattern, in_strValue)  

    # Overwrite file with new content
    try:
       f = open(strFullFileName,'w')
       f.write(strFileContent)
       f.close()
    except IOError:
       log.error("Cannot open file %s", strFullFileName)               
       return            
  
@expose()
def default(self, *args, **kw):
  # redirect to front page
  redirect("/")


  
  
  
