//------------------------------------------------
// Factory to create common fields between 
// call control forms
//------------------------------------------------
Ext.namespace("Paraxip.config.callControl");

function createIdText(config){
  return new Ext.form.TextField(Ext.apply({
    fieldLabel  : "ID",
    name        : "id",
    width       : 162,
    disabled    : true
  }, config));
};

function createNameText(config){
  return new Ext.form.TextField(Ext.apply({
    fieldLabel  : "Display Name",
    name        : "displayName",
    width       : 162
  }, config));
};

function createMinimumOnHookDurationMs(config){
  return new Ext.form.NumberField(Ext.apply({
    fieldLabel    : "Minimum ON-HOOK Duration (ms)",
    name          : "minimumOnHookDurationMs",
    allowNegative : false,
    allowBlank    : false,
    allowDecimals : false
  }, config));
};

function createIsdnTerminationSelection(config){
  return new Ext.form.ComboBox( Ext.apply({
    name            : 'termination',
    fieldLabel      : "Termination",
    helpText        : "isdnCommonParameterstermination",
    store           : ['NETWORK','TERMINAL'],
    forceSelection  : true,
    triggerAction   : 'all',
    mode            : 'local',
    typeAhead       : true,
    selectOnFocus   : true,
    width           : 145,
    minListWidth    : 145
  }, config));
};

// Note: Display name is NET5 but underlying data is set to ETSI.
var variantStore = new Ext.data.SimpleStore({  
  id      : "Paraxip.config.callControl.variantStore",
  fields  : ['value', 'variant_name'],
  data    : [
    ['4ESS', '4ESS'],
    ['5ESS', '5ESS'],    
    ['DMS100', 'DMS100'],
    ['ETSI', 'NET5 (ETSI, EuroISDN)'],          
    ['NI2', 'NI2']     
  ]
});

function createIsdnSwitchVariantSelection(config){
  return new Ext.form.ComboBox( Ext.apply({
    name            : 'switchVariant',
    fieldLabel      : "Switch Variant",
    helpText        : "isdnCommonParametersswitchVariant",
    store           : variantStore,
    displayField    : 'variant_name',
    valueField      : 'value',
    hiddenName      : 'switchVariant',
    forceSelection  : true,
    triggerAction   : 'all',
    mode            : 'local',
    typeAhead       : true,
    selectOnFocus   : true,    
    width           : 145,
    minListWidth    : 145
  }, config));
};

function createIsdnBChannelNegotiationSelection(config){
  return new Ext.form.ComboBox( Ext.apply({
    name            : 'BChannelNegotiation',
    fieldLabel      : "B-Channel Negotiation",
    helpText        : "isdnCommonParametersBChannelNegotiation",
    store           : ['PREFERRED','EXCLUSIVE'],
    forceSelection  : true,
    triggerAction   : 'all',
    mode            : 'local',
    typeAhead       : true,
    selectOnFocus   : true,
    width           : 145,
    minListWidth    : 145
  }, config));
};

function createIsdnInitRestartProcSelection(config){
  return new Ext.form.ComboBox( Ext.apply({
    name            : 'initiateRestartProcedure',
    fieldLabel      : "Initiate Restart Procedure",
    helpText        : "isdnCommonParametersinitiateRestartProcedure",
    store           : ['YES','NO'],
    forceSelection  : true,
    triggerAction   : 'all',
    mode            : 'local',
    typeAhead       : true,
    selectOnFocus   : true,
    width           : 145,
    minListWidth    : 145
  }, config));
};

function createIsdnInProgTonesGenSelection(config){
  return new Ext.form.ComboBox( Ext.apply({
    name            : 'inbandProgressTonesGeneration',
    fieldLabel      : "Inband Progress Tones Generation",
    helpText        : "isdnCommonParametersinbandProgressTonesGeneration",
    store           : ['NEVER','ALWAYS','AS-NEEDED'],
    forceSelection  : true,
    triggerAction   : 'all',
    mode            : 'local',
    typeAhead       : true,
    selectOnFocus   : true,
    width           : 145,
    minListWidth    : 145
  }, config));
};

function createIsdnInProgTonesIndSelection(config){
  return new Ext.form.ComboBox( Ext.apply({
    name            : 'inbandProgressTonesIndicator',
    fieldLabel      : "Inband Progress Tones Indicator",
    helpText        : "isdnCommonParametersinbandProgressTonesIndicator",
    store           : ['ALERTING','PROGRESS','BOTH'],
    forceSelection  : true,
    triggerAction   : 'all',
    mode            : 'local',
    typeAhead       : true,
    selectOnFocus   : true,
    width           : 145,
    minListWidth    : 145
  }, config));
};

function createCallerNameLocationSelection(config){
  return new Ext.form.ComboBox( Ext.apply({
    name            : "callerNameLocation",
    fieldLabel      : "Caller Name Location Method",
    helpText        : "isdnCommonParameterscallerNameLocation",
    store           : ['IN-SETUP-MSG','IN-FACILITY-MSG','DISABLED'],
    forceSelection  : true,
    triggerAction   : 'all',
    mode            : 'local',
    typeAhead       : true,
    selectOnFocus   : true,
    width           : 145,
    minListWidth    : 145
  }, config));
};

function createaWaitFacilityDelayMsNumberField(config){
  return new Ext.form.NumberField( Ext.apply({
    name            : "waitFacilityDelayMs",
    fieldLabel      : "Wait Facility Delay (ms)",
    helpText        : "isdnCommonParameterswaitFacilityDelayMs",
    allowNegative   : false,
    allowBlank      : false,
    allowDecimal    : false,
    width           : 145,
    minListWidth    : 145
  }, config));
};

//------------------------------------------------
//  Common methods
//------------------------------------------------

function callControlCancel(){
  reloadTree(true);
};


